@@ -17,11 +17,6 @@ class Money:
     def amount(self) -> Decimal:
         return self._amount
 
-    def convert_to(self, rate: 'Money') -> 'Money':
-        if self.currency == rate.currency:
-            return Money(self.amount, self.currency)
-        return Money(self.amount * rate.amount, rate.currency)
-
     def round(self, digits=0) -> 'Money':  # noqa: WPS125
         return Money(round(self._amount, digits), self._currency)
 
@@ -53,6 +48,9 @@ class Money:
             raise TypeError(f'different currencies: {self._currency} & {other.currency}')
         return Money(self._amount + other.amount, self._currency)
 
+    def __abs__(self) -> 'Money':
+        return Money(abs(self.amount), self.currency)
+
     def __radd__(self, other) -> 'Money':
         return self.__add__(other)
 
