@@ -2,8 +2,9 @@ import csv
 import datetime
 import logging
 import re
-from typing import Dict, Iterator, List, Tuple
+from typing import Dict, Iterator, List, NamedTuple, Optional, Tuple
 
+from investments.cash import Cash
 from investments.currency import Currency
 from investments.dividend import Dividend
 from investments.fees import Fee
@@ -21,6 +22,10 @@ def _parse_date(strval: str) -> datetime.date:
     return datetime.datetime.strptime(strval, '%Y-%m-%d').date()
 
 
+def _parse_trade_quantity(strval: str) -> int:
+    return int(strval.replace(',', ''))
+
+
 def _parse_dividend_description(description: str) -> Tuple[str, str]:
     m = re.match(r'^(\w+)\s*\(\w+\) (Cash Dividend|Payment in Lieu of Dividend|Choice Dividend)', description)
     if m is None:
@@ -96,25 +101,69 @@ class TickersStorage:
         return self._multipliers[ticker]
 
 
+class SettleDate(NamedTuple):
+    order_id: str
+    settle_date: datetime.date
+
+
+class SettleDatesStorage:
+    def __init__(self):
+        self._settle_data: Dict[Tuple[str, datetime.datetime], SettleDate] = {}
+
+    def __len__(self):
+        return len(self._settle_data)
+
+    def put(
+        self,
+        ticker: str,
+        operation_date: datetime.datetime,
+        settle_date: datetime.date,
+        order_id: str,
+    ):
+        existing_item = self.get(ticker, operation_date)
+        if existing_item:
+            if existing_item.settle_date != settle_date and existing_item.order_id != order_id:
+                raise AssertionError(f'Duplicate settle date for key {(ticker, operation_date)} with {order_id}')
+        self._settle_data[(ticker, operation_date)] = SettleDate(order_id, settle_date)
+
+    def get(
+        self,
+        ticker: str,
+        operation_date: datetime.datetime,
+    ) -> Optional[SettleDate]:
+        return self._settle_data.get((ticker, operation_date))
+
+    def get_date(
+        self,
+        ticker: str,
+        operation_date: datetime.datetime,
+    ) -> Optional[datetime.date]:
+        existing_settle_item = self.get(ticker, operation_date)
+        if existing_settle_item:
+            return existing_settle_item.settle_date
+        return None
+
+
 class InteractiveBrokersReportParser:
     def __init__(self):
         self._trades = []
         self._dividends = []
         self._fees: List[Fee] = []
         self._interests: List[Interest] = []
+        self._cash: List[Cash] = []
         self._deposits_and_withdrawals = []
         self._tickers = TickersStorage()
-        self._settle_dates = {}
+        self._settle_dates = SettleDatesStorage()
 
     def __repr__(self):
         return f'IbParser(trades={len(self.trades)}, dividends={len(self.dividends)}, fees={len(self.fees)}, interests={len(self.interests)})'  # noqa: WPS221
 
     @property
-    def trades(self) -> List:
+    def trades(self) -> List[Trade]:
         return self._trades
 
     @property
-    def dividends(self) -> List:
+    def dividends(self) -> List[Dividend]:
         return self._dividends
 
     @property
@@ -129,6 +178,10 @@ class InteractiveBrokersReportParser:
     def interests(self) -> List[Interest]:
         return self._interests
 
+    @property
+    def cash(self) -> List[Cash]:
+        return self._cash
+
     def parse_csv(self, *, activity_csvs: List[str], trade_confirmation_csvs: List[str]):
         # 1. parse tickers info
         for ac_fname in activity_csvs:
@@ -158,10 +211,11 @@ class InteractiveBrokersReportParser:
                     # 'Mark-to-Market Performance Summary',
                     # 'Net Asset Value', 'Notes/Legal Notes', 'Open Positions', 'Realized & Unrealized Performance Summary',
                     # 'Statement', '\ufeffStatement', 'Total P/L for Statement Period', 'Transaction Fees',
+                    'Cash Report': self._parse_cash_report,
                 })
 
         # 4. sort
-        self._trades.sort(key=lambda x: x.datetime)
+        self._trades.sort(key=lambda x: x.trade_date)
         self._dividends.sort(key=lambda x: x.date)
         self._interests.sort(key=lambda x: x.date)
         self._deposits_and_withdrawals.sort(key=lambda x: x[0])
@@ -174,14 +228,15 @@ class InteractiveBrokersReportParser:
             f = parser.parse(row)
             if f['LevelOfDetail'] != 'EXECUTION':
                 continue
-            settle_date = _parse_date(f['SettleDate'])
+            if f['TransactionType'] == 'TradeCancel':
+                continue
 
-            key = (f['Symbol'], _parse_datetime(f['Date/Time']))
-            existing_settle_date = self._settle_dates.get(key)
-            if existing_settle_date is not None:
-                assert existing_settle_date == settle_date
-            else:
-                self._settle_dates[key] = settle_date
+            self._settle_dates.put(
+                f['Symbol'],
+                _parse_datetime(f['Date/Time']),
+                _parse_date(f['SettleDate']),
+                f['OrderID'],
+            )
 
     def _real_parse_activity_csv(self, csv_reader: Iterator[List[str]], parsers):
         nrparser = NamedRowsParser()
@@ -226,14 +281,14 @@ class InteractiveBrokersReportParser:
 
         dt = _parse_datetime(f['Date/Time'])
 
-        settle_date = self._settle_dates.get((ticker.symbol, dt))
+        settle_date = self._settle_dates.get_date(ticker.symbol, dt)
         assert settle_date is not None
 
         self._trades.append(Trade(
             ticker=ticker,
-            datetime=dt,
+            trade_date=dt,
             settle_date=settle_date,
-            quantity=int(f['Quantity']) * quantity_multiplier,
+            quantity=_parse_trade_quantity(f['Quantity']) * quantity_multiplier,
             price=Money(f['T. Price'], currency),
             fee=Money(f['Comm/Fee'], currency),
         ))
@@ -313,3 +368,11 @@ class InteractiveBrokersReportParser:
         amount = Money(f['Amount'], currency)
         description = f['Description']
         self._interests.append(Interest(date, amount, description))
+
+    def _parse_cash_report(self, f: Dict[str, str]):
+        currency_code = f['Currency']
+        if currency_code != 'Base Currency Summary':
+            currency = Currency.parse(currency_code)
+            description = f['Currency Summary']
+            amount = Money(f['Total'], currency)
+            self._cash.append(Cash(description, amount))
