@@ -1,6 +1,6 @@
 import datetime
 import re
-import xml.etree.ElementTree as ET  # type: ignore
+import xml.etree.ElementTree as ET  # noqa:N817
 from typing import List, Optional
 
 from investments.currency import Currency
@@ -21,6 +21,8 @@ def _parse_tickerkind(strval: str):
         return TickerKind.Rdr
     if strval == 'Облигации':
         return TickerKind.Bond
+    if strval == 'GDR':
+        return TickerKind.Gdr
     raise ValueError(strval)
 
 
@@ -98,13 +100,15 @@ class OpenBrokerFRParser:
         self._parse_non_trade_operations(tree)
         self._parse_trades(tree)
 
-        self._trades.sort(key=lambda x: x.datetime)
+        self._trades.sort(key=lambda x: x.trade_date)
         self._dividends.sort(key=lambda x: x.date)
         self._deposits_and_withdrawals.sort(key=lambda x: x[0])
 
     def _parse_tickers(self, xml_tree: ET.ElementTree):
         for rec in xml_tree.findall('spot_portfolio_security_params/item'):
             f = rec.attrib
+            if 'ticker' not in f and f['isin'] == 'JE00B5BCW814':
+                f['ticker'] = 'RUAL'
             self._tickers.put(
                 symbol=f['ticker'],
                 kind=_parse_tickerkind(f['security_type']),
@@ -115,16 +119,16 @@ class OpenBrokerFRParser:
 
     def _parse_cb_convertation(self, f):
         # WARNING: lost price information, do not use for tax calculation!
-        qnty = int(f['quantity'])
+        qnty = int(float(f['quantity']))
         assert float(f['quantity']) == qnty
         ticker = self._tickers.get(name=f['security_name'])
         dt = _parse_datetime(f['operation_date'])
         self._trades.append(Trade(
             ticker=ticker,
-            datetime=dt,
+            trade_date=dt,
             settle_date=dt,
             quantity=qnty,
-            price=Money(0, Currency.RUB),  # TODO: other currencies
+            price=Money(0, Currency.RUB),
             fee=Money(0, Currency.RUB),
         ))
 
@@ -155,7 +159,7 @@ class OpenBrokerFRParser:
             isin, quantity_buyout = m.group('isin'), int(m.group('quantity'))
             self._trades.append(Trade(
                 ticker=self._tickers.get(isin=isin),
-                datetime=dt,
+                trade_date=dt,
                 settle_date=dt,
                 quantity=-1 * quantity_buyout,
                 price=money_total / quantity_buyout,
@@ -171,7 +175,7 @@ class OpenBrokerFRParser:
                 for (price, quantity_coupons) in ((Money(0, currency), 1), (money_total, -1)):
                     self._trades.append(Trade(
                         ticker=ticker,
-                        datetime=dt,
+                        trade_date=dt,
                         settle_date=dt,
                         quantity=quantity_coupons,
                         price=price,
@@ -184,7 +188,7 @@ class OpenBrokerFRParser:
                 quantity_redemption = bonds_redemption[key]
                 self._trades.append(Trade(
                     ticker=ticker,
-                    datetime=dt,
+                    trade_date=dt,
                     settle_date=dt,
                     quantity=quantity_redemption,
                     price=-1 * money_total / int(quantity_redemption),
@@ -275,7 +279,7 @@ class OpenBrokerFRParser:
 
             self._trades.append(Trade(
                 ticker=ticker,
-                datetime=_parse_datetime(f['conclusion_time']),
+                trade_date=_parse_datetime(f['conclusion_time']),
                 settle_date=_parse_datetime(f['execution_date']),
                 quantity=int(qnty),
                 price=price,
